body {
  background: #fff;
}

#header {
  width: 950px;
  margin: 0 auto;
  height: 102px;
}

#navigation {
  display: block;
  float: left;
  margin: 27px 0 0 25px;
  padding: 0;
}

#navigation li{
  float: left;
  display: block;
  margin-right: 25px;
}

#navigation li a {
  display: block;
  font-weight: normal;
  text-decoration: none;
  background-position: 50% 0;
  padding: 20px 0 5px;
  color: #353535;
  font-size: 14px;
}

#navigation li a.current, #navigation li a.section {
  border-bottom: 3px solid #cf2f19;
  color: #cf2f19;
}

div.related {
  background-color: #cde2f8;
  border: 1px solid #b0d3f8;
}

div.related a {
  color: #4078ba;
  text-shadow: none;
}

div.sphinxsidebarwrapper {
  padding-top: 0;
}

pre {
  color: #555;
}

div.documentwrapper h1, div.documentwrapper h2, div.documentwrapper h3, div.documentwrapper h4, div.documentwrapper h5, div.documentwrapper h6 {
  font-family: 'PT Sans', sans-serif !important;
  color: #264D69;
  border-bottom: 1px dotted #C5E2EA;
  padding: 0;
  background: none;
  padding-bottom: 5px;
}

div.documentwrapper h3 {
  color: #CF2F19;
}

a.headerlink {
  color: #fff !important;
  margin-left: 5px;
  background: #CF2F19 !important;
}

div.body {
  margin-left: 230px;
}

div.document {
  width: 960px;
  margin: 0 auto;
}

a.reference.internal em {
  font-style: normal;
}