# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

if __name__ == '__main__':

    # open file 'hadoop-job-file.txt' for write and read
    f = open('hadoop-job-file.txt', 'wr')

    s = '1234567890'

    i = 0
    j = 0

    # strings number in file
    str_number = 1000000

    # repetitions number 's' in each string of file
    rep_number = 10

    while i < str_number:
        while j < rep_number:
            f.write(s)
            j += 1
        f.write('\n')
        i += 1
        j = 0

    f.close()
